//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

using MindFusion.Diagramming.Layout;


namespace MindFusion.Diagramming.WinForms.Samples.CS.ClusterLayout
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

			diagram1.DefaultShape = Shapes.Ellipse;
			diagram1.LinkHeadShape = ArrowHeads.PointerArrow;
			diagram1.LinkHeadShapeSize = 4;
			diagram1.ShadowsStyle = ShadowsStyle.None;

			diagram1.ShapeNodeStyle.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(156, 170, 198));
			diagram1.ShapeNodeStyle.Stroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(97, 106, 127));

			diagram1.DiagramLinkStyle.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(97, 106, 127));
			diagram1.DiagramLinkStyle.Stroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(192, 192, 192));
			diagram1.DiagramLinkStyle.HeadStroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(192, 192, 192));
		}

		private void button1_Click(object sender, EventArgs e)
        {
            RandomGraph();
            Arrange();
        }

        private void Arrange()
        {
			var layout = new Layout.CompositeLayout();
			layout.RotateSubgraphs = true;
			layout.PartitionMethod = MindFusion.Layout.PartitionMethod.LinkDensity;

			var subgraphLayout = new CircularLayout();
			subgraphLayout.Radius = 0;
			subgraphLayout.SplitGraph = false;
			layout.SubgraphLayout = subgraphLayout;

			var masterLayout = new AnnealLayout();
			masterLayout.CrossingLinksCost = 0;
			layout.MasterLayout = masterLayout;

			layout.Arrange(diagram1);

			diagram1.ResizeToFitItems(5);
		}

		private void RandomGraph()
        {
			diagram1.ClearAll();

			var rnd = new Random();
			int numClusters = rnd.Next(4, 8);
			var clusters = new List<List<DiagramNode>>();
			for (int i = 0; i < numClusters; i++)
			{
				int clusterSize = rnd.Next(10, 18);
				var cluster = CreateCluster(clusterSize);
				clusters.Add(cluster);
			}

			for (int i = 0; i < numClusters; i++)
			{
				var cluster1 = clusters[i];
				var cluster2 = clusters[(i + 1) % numClusters];
				var cluster3 = clusters[(i + rnd.Next(1, numClusters - 1)) % numClusters];
				var cluster4 = clusters[(i + rnd.Next(1, numClusters - 1)) % numClusters];

				var node1 = cluster1[rnd.Next(cluster1.Count)];
				var node2 = cluster2[rnd.Next(cluster2.Count)];
				var node3 = cluster3[rnd.Next(cluster3.Count)];
				var node4 = cluster4[rnd.Next(cluster4.Count)];

				diagram1.Factory.CreateDiagramLink(node1, node2);
				diagram1.Factory.CreateDiagramLink(node1, node3);
				diagram1.Factory.CreateDiagramLink(node1, node4);
			}
		}

		List<DiagramNode> CreateCluster(int numNodes)
		{
			var cluster = new List<DiagramNode>();
			for (int n = 0; n < numNodes; ++n)
			{
				var node = diagram1.Factory.CreateShapeNode(0, 0, 10, 10);
				cluster.Add(node);
			}

			for (int i = 0; i < numNodes; i++)
			{
				for (int j = i + 1; j < numNodes; j++)
				{
					var node1 = cluster[i];
					var node2 = cluster[j];
					diagram1.Factory.CreateDiagramLink(node1, node2);
				}
			}

			return cluster;
		}
    }
}
